@@ -2,6 +2,8 @@
    fdisk.h
 */
 
+#include "c.h"
+
 #define DEFAULT_SECTOR_SIZE	512
 #define MAX_SECTOR_SIZE	2048
 #define SECTOR_SIZE	512	/* still used in BSD code */
@@ -21,14 +23,9 @@
 #define IS_EXTENDED(i) \
 	((i) == EXTENDED || (i) == WIN98_EXTENDED || (i) == LINUX_EXTENDED)
 
-#define SIZE(a)	(sizeof(a)/sizeof((a)[0]))
-
 #define cround(n)	(display_in_cyl_units ? ((n)/units_per_sector)+1 : (n))
 #define scround(x)	(((x)+units_per_sector-1)/units_per_sector)
 
-extern long long ext2_llseek(unsigned int fd, long long offset,
-			     unsigned int origin);
-
 struct partition {
 	unsigned char boot_ind;         /* 0x80 - active */
 	unsigned char head;             /* starting head */
@@ -40,13 +37,18 @@ struct partition {
 	unsigned char end_cyl;          /* end cylinder */
 	unsigned char start4[4];        /* starting sector counting from 0 */
 	unsigned char size4[4];         /* nr of sectors in partition */
+} __attribute__ ((packed));
+
+enum menutype {
+	MAIN_MENU,
+	EXPERT_MENU,
 };
 
-enum failure {usage, usage2, ioctl_error,
+enum failure {ioctl_error,
 	unable_to_open, unable_to_read, unable_to_seek,
-	unable_to_write, out_of_memory};
+	unable_to_write};
 
-enum action {fdisk, require, try_only, create_empty_dos, create_empty_sun};
+enum action {fdisk, try_only};
 
 struct geom {
 	unsigned int heads;
@@ -61,10 +63,9 @@ extern unsigned int display_in_cyl_units, units_per_sector;
 extern void change_units(void);
 extern void fatal(enum failure why);
 extern void get_geometry(int fd, struct geom *);
-extern int get_boot(enum action what);
 extern int  get_partition(int warn, int max);
 extern void list_types(struct systypes *sys);
-extern int read_line (void);
+extern int read_line (int *asked);
 extern char read_char(char *mesg);
 extern int read_hex(struct systypes *sys);
 extern void reread_partition_table(int leave);
@@ -72,24 +73,41 @@ extern struct partition *get_part_table(int);
 extern int valid_part_table_flag(unsigned char *b);
 extern unsigned int read_int(unsigned int low, unsigned int dflt,
 			     unsigned int high, unsigned int base, char *mesg);
+extern void print_menu(enum menutype);
+extern void print_partition_size(int num, unsigned long long start, unsigned long long stop, int sysid);
 
+extern unsigned char *MBRbuffer;
+extern void zeroize_mbr_buffer(void);
+
+extern unsigned int heads, cylinders, sector_size;
+extern unsigned long long sectors;
+extern char *partition_type(unsigned char type);
+extern void update_units(void);
+extern char read_chars(char *mesg);
+extern void set_changed(int);
+extern void set_all_unchanged(void);
 
 #define PLURAL	0
 #define SINGULAR 1
-extern char *const str_units(int);
+extern const char * str_units(int);
 
-extern unsigned int get_start_sect(struct partition *p);
-extern unsigned int get_nr_sects(struct partition *p);
+extern unsigned long long get_start_sect(struct partition *p);
+extern unsigned long long get_nr_sects(struct partition *p);
 
-extern int osf_label;
+enum labeltype {
+	DOS_LABEL = 1,
+	SUN_LABEL = 2,
+	SGI_LABEL = 4,
+	AIX_LABEL = 8,
+	OSF_LABEL = 16,
+	MAC_LABEL = 32,
+	ANY_LABEL = -1
+};
+
+extern enum labeltype disklabel;
 
 /* prototypes for fdiskbsdlabel.c */
-extern void bselect(void);
+extern void bsd_command_prompt(void);
 extern int check_osf_label(void);
 extern int btrydev(char * dev);
 extern void xbsd_print_disklabel(int);
-
-/* prototypes for fdisksgilabel.c */
-extern int valid_part_table_flag(unsigned char *b);
-
-#define PROC_PARTITIONS "/proc/partitions"
