@@ -40,27 +40,26 @@
 
 /*
  * look -- find lines in a sorted list.
- * 
+ *
  * The man page said that TABs and SPACEs participate in -d comparisons.
  * In fact, they were ignored.  This implements historic practice, not
  * the manual page.
  */
 
-#include <sys/types.h>
 #include <sys/mman.h>
 #include <sys/stat.h>
-
-#include <limits.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stddef.h>
 #include <string.h>
 #include <ctype.h>
 #include <getopt.h>
-#include <locale.h>
-#include "pathnames.h"
+
 #include "nls.h"
+#include "xalloc.h"
+#include "pathnames.h"
 
 #define	EQUAL		0
 #define	GREATER		1
@@ -75,11 +74,10 @@ char *comparbuf;
 
 static char *binary_search (char *, char *);
 static int compare (char *, char *);
-static void err (const char *fmt, ...);
 static char *linear_search (char *, char *);
 static int look (char *, char *);
 static void print_from (char *, char *);
-static void usage (void);
+static void __attribute__ ((__noreturn__)) usage(FILE * out);
 
 int
 main(int argc, char *argv[])
@@ -88,21 +86,31 @@ main(int argc, char *argv[])
 	int ch, fd, termchar;
 	char *back, *file, *front, *p;
 
+	static const struct option longopts[] = {
+		{"alternative", no_argument, NULL, 'a'},
+		{"alphanum", no_argument, NULL, 'd'},
+		{"ignore-case", no_argument, NULL, 'f'},
+		{"terminate", required_argument, NULL, 't'},
+		{"version", no_argument, NULL, 'V'},
+		{"help", no_argument, NULL, 'h'},
+		{NULL, 0, NULL, 0}
+	};
+
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
-	
+
 	setlocale(LC_ALL, "");
 
 	file = _PATH_WORDS;
 	termchar = '\0';
 	string = NULL;		/* just for gcc */
 
-	while ((ch = getopt(argc, argv, "adft:")) != -1)
+	while ((ch = getopt_long(argc, argv, "adft:Vh", longopts, NULL)) != -1)
 		switch(ch) {
 		case 'a':
-   		        file = _PATH_WORDS_ALT;
-		        break;
+			file = _PATH_WORDS_ALT;
+			break;
 		case 'd':
 			dflag = 1;
 			break;
@@ -112,9 +120,16 @@ main(int argc, char *argv[])
 		case 't':
 			termchar = *optarg;
 			break;
+		case 'V':
+			printf(_("%s from %s\n"),
+				program_invocation_short_name,
+				PACKAGE_STRING);
+			return EXIT_SUCCESS;
+		case 'h':
+			usage(stdout);
 		case '?':
 		default:
-			usage();
+			usage(stderr);
 		}
 	argc -= optind;
 	argv += optind;
@@ -129,14 +144,14 @@ main(int argc, char *argv[])
 		string = *argv;
 		break;
 	default:
-		usage();
+		usage(stderr);
 	}
 
 	if (termchar != '\0' && (p = strchr(string, termchar)) != NULL)
 		*++p = '\0';
 
 	if ((fd = open(file, O_RDONLY, 0)) < 0 || fstat(fd, &sb))
-		err("%s: %s", file, strerror(errno));
+		err(EXIT_FAILURE, "%s", file);
 	front = mmap(NULL, (size_t) sb.st_size, PROT_READ,
 #ifdef MAP_FILE
 		     MAP_FILE |
@@ -144,11 +159,11 @@ main(int argc, char *argv[])
 		     MAP_SHARED, fd, (off_t) 0);
 	if
 #ifdef MAP_FAILED
-	   (front == MAP_FAILED)
+		(front == MAP_FAILED)
 #else
-	   ((void *)(front) <= (void *)0)
+		((void *)(front) <= (void *)0)
 #endif
-		err("%s: %s", file, strerror(errno));
+			err(EXIT_FAILURE, "%s", file);
 
 #if 0
 	/* workaround for mmap problem (rmiller@duskglow.com) */
@@ -177,51 +192,52 @@ look(char *front, char *back)
 	} else
 		stringlen = strlen(string);
 
-	comparbuf = malloc(stringlen+1);
-	if (comparbuf == NULL)
-		err(_("Out of memory"));
+	comparbuf = xmalloc(stringlen+1);
 
 	front = binary_search(front, back);
 	front = linear_search(front, back);
 
 	if (front)
 		print_from(front, back);
+
+	free(comparbuf);
+
 	return (front ? 0 : 1);
 }
 
 
 /*
  * Binary search for "string" in memory between "front" and "back".
- * 
+ *
  * This routine is expected to return a pointer to the start of a line at
  * *or before* the first word matching "string".  Relaxing the constraint
  * this way simplifies the algorithm.
- * 
+ *
  * Invariants:
- * 	front points to the beginning of a line at or before the first 
+ * 	front points to the beginning of a line at or before the first
  *	matching string.
- * 
- * 	back points to the beginning of a line at or after the first 
+ *
+ * 	back points to the beginning of a line at or after the first
  *	matching line.
- * 
+ *
  * Advancing the Invariants:
- * 
+ *
  * 	p = first newline after halfway point from front to back.
- * 
- * 	If the string at "p" is not greater than the string to match, 
+ *
+ * 	If the string at "p" is not greater than the string to match,
  *	p is the new front.  Otherwise it is the new back.
- * 
+ *
  * Termination:
- * 
- * 	The definition of the routine allows it return at any point, 
+ *
+ * 	The definition of the routine allows it return at any point,
  *	since front is always at or before the line to print.
- * 
- * 	In fact, it returns when the chosen "p" equals "back".  This 
- *	implies that there exists a string is least half as long as 
- *	(back - front), which in turn implies that a linear search will 
+ *
+ * 	In fact, it returns when the chosen "p" equals "back".  This
+ *	implies that there exists a string is least half as long as
+ *	(back - front), which in turn implies that a linear search will
  *	be no more expensive than the cost of simply printing a string or two.
- * 
- * 	Trying to continue with binary search at this point would be 
+ *
+ * 	Trying to continue with binary search at this point would be
  *	more trouble than it's worth.
  */
 #define	SKIP_PAST_NEWLINE(p, back) \
@@ -253,12 +269,12 @@ binary_search(char *front, char *back)
 /*
  * Find the first line that starts with string, linearly searching from front
  * to back.
- * 
+ *
  * Return NULL for no such line.
- * 
+ *
  * This routine assumes:
- * 
- * 	o front points at the first character in a line. 
+ *
+ * 	o front points at the first character in a line.
  *	o front is before or at the first line to be printed.
  */
 char *
@@ -283,7 +299,7 @@ linear_search(char *front, char *back)
 /*
  * Print as many lines as match string, starting at front.
  */
-void 
+void
 print_from(char *front, char *back)
 {
 	int eol;
@@ -293,7 +309,7 @@ print_from(char *front, char *back)
 			eol = 0;
 			while (front < back && !eol) {
 				if (putchar(*front) == EOF)
-					err("stdout: %s", strerror(errno));
+					err(EXIT_FAILURE, "stdout");
 				if (*front++ == '\n')
 					eol = 1;
 			}
@@ -305,13 +321,13 @@ print_from(char *front, char *back)
 /*
  * Return LESS, GREATER, or EQUAL depending on how  string  compares with
  * string2 (s1 ??? s2).
- * 
- * 	o Matches up to len(s1) are EQUAL. 
+ *
+ * 	o Matches up to len(s1) are EQUAL.
  *	o Matches up to len(s2) are GREATER.
- * 
+ *
  * Compare understands about the -f and -d flags, and treats comparisons
  * appropriately.
- * 
+ *
  * The string "string" is null terminated.  The string "s2" is '\n' terminated
  * (or "s2end" terminated).
  *
@@ -326,9 +342,12 @@ compare(char *s2, char *s2end) {
 	/* copy, ignoring things that should be ignored */
 	p = comparbuf;
 	i = stringlen;
-	while(s2 < s2end && *s2 != '\n' && i--) {
+	while(s2 < s2end && *s2 != '\n' && i) {
 		if (!dflag || isalnum(*s2))
+		{
 			*p++ = *s2;
+			i--;
+		}
 		s2++;
 	}
 	*p = 0;
@@ -342,38 +361,19 @@ compare(char *s2, char *s2end) {
 	return ((i > 0) ? LESS : (i < 0) ? GREATER : EQUAL);
 }
 
-static void
-usage()
+static void __attribute__ ((__noreturn__)) usage(FILE * out)
 {
-	(void)fprintf(stderr, _("usage: look [-dfa] [-t char] string [file]\n"));
-	exit(2);
-}
+	fputs(_("\nUsage:\n"), out),
+	fprintf(out,
+	      _(" %s [options] string [file]\n"), program_invocation_short_name);
 
-#if __STDC__
-#include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
+	fputs(_("\nOptions:\n"), out);
+	fputs(_(" -a, --alternative      use alternate dictionary\n"
+		" -d, --alphanum         compare only alpha numeric characters\n"
+		" -f, --ignore-case      ignore when comparing\n"
+		" -t, --terminate <char> define string termination character\n"
+		" -V, --version          output version information and exit\n"
+		" -h, --help             display this help and exit\n\n"), out);
 
-void
-#if __STDC__
-err(const char *fmt, ...)
-#else
-err(fmt, va_alist)
-	char *fmt;
-	va_dcl
-#endif
-{
-	va_list ap;
-#if __STDC__
-	va_start(ap, fmt);
-#else
-	va_start(ap);
-#endif
-	(void)fprintf(stderr, "look: ");
-	(void)vfprintf(stderr, fmt, ap);
-	va_end(ap);
-	(void)fprintf(stderr, "\n");
-	exit(2);
-	/* NOTREACHED */
+	exit(out == stderr ? EXIT_FAILURE : EXIT_SUCCESS);
 }
